(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[6355],{7907:function(e,r,n){"use strict";n.r(r),n.d(r,{default:function(){return y}});var t=n(67294),c=n(59499),o=n(50029),u=n(16835),i=n(87794),s=n.n(i),a=n(73662),f=n(11163),p=n(77979),w=n(97142),O=n(45035),b=n(43072),l=n(85893);function j(e,r){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var t=Object.getOwnPropertySymbols(e);r&&(t=t.filter((function(r){return Object.getOwnPropertyDescriptor(e,r).enumerable}))),n.push.apply(n,t)}return n}function v(e){for(var r=1;r<arguments.length;r++){var n=null!=arguments[r]?arguments[r]:{};r%2?j(Object(n),!0).forEach((function(r){(0,c.Z)(e,r,n[r])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):j(Object(n)).forEach((function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))}))}return e}var d=function(){var e=(0,f.useRouter)(),r=(0,b.ny)(),n=(0,u.Z)(r,1)[0],c=(0,w.T)();(0,t.useEffect)((function(){c((0,b.Vv)(void 0))}),[c]);var i=function(){var r=(0,o.Z)(s().mark((function r(t){var c,o;return s().wrap((function(r){for(;;)switch(r.prev=r.next){case 0:return c=(0,a.JJ)(t.password),r.next=3,n(v(v({},t),{},{password:c})).unwrap();case 3:o=r.sent,e.push("".concat(O.e3,"/profile/").concat(o.id));case 5:case"end":return r.stop()}}),r)})));return function(e){return r.apply(this,arguments)}}();return(0,l.jsx)("div",{children:(0,l.jsx)("main",{children:(0,l.jsx)(p.Z,{onSubmit:i})})})},h=n(33278),y=function(){return(0,l.jsx)(h.Z,{title:"New User",children:(0,l.jsx)(d,{})})}},51050:function(e,r,n){(window.__NEXT_P=window.__NEXT_P||[]).push(["/user-management/new",function(){return n(7907)}])}},function(e){e.O(0,[2437,3880,4231,2495,3879,8686,5363,9774,2888,179],(function(){return r=51050,e(e.s=r);var r}));var r=e.O();_N_E=r}]);