self.__BUILD_MANIFEST=function(s,a,c,e,t,n,i,d,u,r,o,f,g,p,b,m,h,k,j,y){return{__rewrites:{beforeFiles:[],afterFiles:[{source:"/api/v1/:path"},{source:"/api/v1/:first/:second*"},{source:"/health"}],fallback:[]},"/":[s,"static/chunks/pages/index-a6a695991216e778.js"],"/404":["static/chunks/pages/404-20745ea77433b998.js"],"/_error":["static/chunks/pages/_error-cd8de5d864b8d699.js"],"/add-systems":[s,a,e,f,c,"static/chunks/pages/add-systems-dac52b3f0cb544c8.js"],"/add-systems/manual":[s,a,e,t,n,i,c,d,g,"static/chunks/pages/add-systems/manual-51b16e021db36021.js"],"/classify-systems":[s,f,p,"static/chunks/pages/classify-systems-6269057c83460a8f.js"],"/consent/privacy-notices":[s,b,"static/chunks/pages/consent/privacy-notices-481f7aa077601f97.js"],"/consent/privacy-notices/[id]":[s,"static/chunks/pages/consent/privacy-notices/[id]-94f2406364423c89.js"],"/datamap":["static/chunks/674a26a7-89baf463c0a5e700.js",s,a,e,b,"static/chunks/6904-9512de1fe60459df.js",c,"static/chunks/pages/datamap-2b77bed050483143.js"],"/dataset":[s,"static/chunks/4995-a3585b8c943580a3.js",p,"static/chunks/pages/dataset-50fe1b44a634fd7a.js"],"/dataset/new":[s,a,e,u,c,"static/chunks/pages/dataset/new-16f26077242ec5d9.js"],"/dataset/[id]":[s,a,n,c,"static/chunks/pages/dataset/[id]-054cdbdc9eee52c5.js"],"/datastore-connection":[s,"static/css/684ec0de717c9d2d.css","static/chunks/pages/datastore-connection-ce088f5a768737d5.js"],"/datastore-connection/new":[s,e,t,u,m,h,"static/chunks/pages/datastore-connection/new-a09e629361fab14c.js"],"/datastore-connection/[id]":[s,e,t,u,m,h,"static/chunks/pages/datastore-connection/[id]-faba90c0fd922c7d.js"],"/login":["static/chunks/pages/login-196778433a92986b.js"],"/management/about":[s,"static/chunks/pages/management/about-3125233231a6a782.js"],"/management/organization":[s,a,e,c,"static/chunks/pages/management/organization-d29bfbfb8084db21.js"],"/privacy-requests":[s,a,e,"static/chunks/3419-0ec136bc8f66fda5.js",c,k,"static/chunks/pages/privacy-requests-e5dead3ff0b0b678.js"],"/privacy-requests/configure":[s,"static/chunks/pages/privacy-requests/configure-b47b461ebb9cb89e.js"],"/privacy-requests/configure/messaging":[s,a,c,"static/chunks/pages/privacy-requests/configure/messaging-a01c9297b3a633c3.js"],"/privacy-requests/configure/storage":[s,a,c,"static/chunks/pages/privacy-requests/configure/storage-65f4adc818ee224c.js"],"/privacy-requests/[id]":[s,a,e,c,k,"static/chunks/pages/privacy-requests/[id]-bdc93f509c857290.js"],"/systems":[s,r,o,"static/chunks/pages/systems-318a3841395102c9.js"],"/systems/configure/[id]":[s,a,e,t,n,i,c,d,g,"static/chunks/pages/systems/configure/[id]-7f4f4cf81b266bcb.js"],"/taxonomy":[s,a,e,t,i,c,d,"static/chunks/pages/taxonomy-71118098bf5c64ad.js"],"/user-management":[s,a,e,c,"static/chunks/pages/user-management-668bff11237932f9.js"],"/user-management/new":[s,a,e,j,c,r,y,o,"static/chunks/pages/user-management/new-a6bc85ce1f009bb7.js"],"/user-management/profile/[id]":[s,a,e,j,c,r,y,o,"static/chunks/pages/user-management/profile/[id]-c89b91e39923ffa2.js"],sortedPages:["/","/404","/_app","/_error","/add-systems","/add-systems/manual","/classify-systems","/consent/privacy-notices","/consent/privacy-notices/[id]","/datamap","/dataset","/dataset/new","/dataset/[id]","/datastore-connection","/datastore-connection/new","/datastore-connection/[id]","/login","/management/about","/management/organization","/privacy-requests","/privacy-requests/configure","/privacy-requests/configure/messaging","/privacy-requests/configure/storage","/privacy-requests/[id]","/systems","/systems/configure/[id]","/taxonomy","/user-management","/user-management/new","/user-management/profile/[id]"]}}("static/chunks/2437-346b800ba1e7260a.js","static/chunks/3880-9cb5b7e95738d343.js","static/chunks/3879-d93b4c6f00a60acd.js","static/chunks/4231-030e51971071e2d4.js","static/chunks/6114-5d38dd187fbe66bc.js","static/chunks/9425-57dc5c731718e34c.js","static/chunks/1012-30aa504ee48d2ac0.js","static/chunks/1925-a7376af6d6b3ce18.js","static/chunks/6647-38ce0c538b0ccf6f.js","static/chunks/8686-c4754f0f613627d4.js","static/css/670d677a9586d067.css","static/chunks/5738-4c854c27c4593e75.js","static/chunks/4940-472ebd51d5c62600.js","static/chunks/4409-f6e492eff1d25dc7.js","static/chunks/8104-0d4fc1fc38c7c284.js","static/chunks/7826-caefbfdb091ebeef.js","static/chunks/6459-917fb5ecd036df8b.js","static/chunks/6765-5b2b0aa734a68298.js","static/chunks/2495-f129e9c464fa7cd2.js","static/chunks/5363-e40e5b442dd840bc.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();