(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3691],{30042:function(n,e,i){"use strict";i.r(e),i.d(e,{default:function(){return j}});var s=i(49609),t=i(68527),r=i(11739),u=i(41664),o=i.n(u),d=i(11163),c=i(67294),f=i(97142),h=i(18343),l=i(45035),a=i(67087),x=i(64940),y=i(85893),m=function(){return(0,y.jsx)(t.gC,{alignItems:"stretch",flex:"1",gap:"18px",maxWidth:"70vw",children:(0,y.jsx)(x.Z,{})})},j=function(){var n=(0,d.useRouter)(),e=(0,f.T)(),i="";n.query.id&&(i=Array.isArray(n.query.id)?n.query.id[0]:n.query.id);var u=(0,a.rn)(i,{skip:!i}),x=u.data,j=u.isLoading;return(0,c.useEffect)((function(){e((0,a.db)(x))}),[x,e]),j?(0,y.jsx)(h.Z,{title:"Systems",children:(0,y.jsx)(s.$,{})}):(0,y.jsxs)(h.Z,{title:"Systems",children:[(0,y.jsx)(t.X6,{mb:2,fontSize:"2xl",fontWeight:"semibold",children:"Configure your system"}),(0,y.jsx)(t.xu,{mb:8,children:(0,y.jsxs)(r.aG,{fontWeight:"medium",fontSize:"sm",color:"gray.600",children:[(0,y.jsx)(r.gN,{children:(0,y.jsx)(o(),{href:l.So,children:"System Connections"})}),(0,y.jsx)(r.gN,{children:(0,y.jsx)(o(),{href:"#",children:"Configure your connection"})})]})}),x?(0,y.jsx)(m,{}):(0,y.jsxs)(t.xv,{"data-testid":"system-not-found",children:["Could not find a system with id ",i]})]})}},88420:function(n,e,i){(window.__NEXT_P=window.__NEXT_P||[]).push(["/systems/configure/[id]",function(){return i(30042)}])}},function(n){n.O(0,[2437,3880,4231,6114,9425,1012,3879,1925,4940,9774,2888,179],(function(){return e=88420,n(n.s=e);var e}));var e=n.O();_N_E=e}]);